@@ -2,26 +2,29 @@ package Test2::Formatter::Stream;
 use strict;
 use warnings;
 
-our $VERSION = '0.001006'; # TRIAL
+our $VERSION = '0.001051'; # TRIAL
 
-use Carp qw/croak/;
+use Carp qw/croak confess/;
 use Time::HiRes qw/time/;
+use IO::Handle;
 
 use Test2::Harness::Util::JSON qw/JSON/;
 
 use base qw/Test2::Formatter/;
 use Test2::Util::HashBase qw/-io _encoding _no_header _no_numbers _no_diag -event_id -tb -tb_handles -file -leader/;
 
-{
+BEGIN {
     my $J = JSON->new;
     $J->indent(0);
     $J->convert_blessed(1);
     $J->allow_blessed(1);
     $J->utf8(1);
 
-    sub ENCODER() { $J }
+    require constant;
+    constant->import(ENCODER => $J);
 }
 
+my $ROOT_PID;
 my $ROOT_FILE;
 sub import {
     my $class = shift;
@@ -29,6 +32,7 @@ sub import {
 
     $class->SUPER::import();
 
+    $ROOT_PID  = $$;
     $ROOT_FILE = $params{file} if $params{file};
 }
 
@@ -40,6 +44,7 @@ sub init {
     $self->{+EVENT_ID} = 1;
 
     if (my $file = $self->{+FILE}) {
+        confess "File '$file' already exists!" if -f $file;
         open(my $fh, '>', $file) or die "Could not open file: $file";
         $fh->autoflush(1);
         unshift @{$self->{+IO}} => $fh;
@@ -71,6 +76,9 @@ sub new_root {
     my $class = shift;
     my %params = @_;
 
+    confess "new_root called from child process!"
+        if $ROOT_PID != $$;
+
     require Test2::API;
     my $io = $params{+IO} = [Test2::API::test2_stdout(), Test2::API::test2_stderr()];
     $_->autoflush(1) for @$io;
@@ -98,6 +106,7 @@ sub record {
         $json = ENCODER->encode(
             {
                 stamp        => time,
+                times        => [times],
                 stream_id    => $id,
                 event_id     => "event-$id",
                 facet_data   => $facets,
