@@ -2,13 +2,18 @@ package Test2::Formatter::Test2;
 use strict;
 use warnings;
 
-our $VERSION = '0.001006'; # TRIAL
+our $VERSION = '0.001051'; # TRIAL
 
 use Scalar::Util qw/blessed/;
-use List::Util qw/shuffle/;
+use List::Util qw/shuffle first/;
 use Test2::Util::Term qw/term_size/;
 use Test2::Harness::Util::Term qw/USE_ANSI_COLOR/;
-use Test2::Util qw/IS_WIN32/;
+use Test2::Util qw/IS_WIN32 clone_io/;
+use Time::HiRes;
+use IO::Handle;
+
+use File::Spec();
+use Test2::Formatter::Test2::Composer;
 
 BEGIN { require Test2::Formatter; our @ISA = qw(Test2::Formatter) }
 
@@ -19,6 +24,7 @@ sub import {
 }
 
 use Test2::Util::HashBase qw{
+    -composer
     -last_depth
     -_buffered
     -io
@@ -30,6 +36,8 @@ use Test2::Util::HashBase qw{
     -job_length
     -ecount
     -job_colors
+    -active_files
+    -_active_disp
 };
 
 sub TAG_WIDTH() { 8 }
@@ -121,33 +129,46 @@ sub DEFAULT_COLOR() {
     );
 }
 
+my %FACET_TAG_BORDERS = (
+    'default' => ['[', ']'],
+    'amnesty' => ['{', '}'],
+    'info'    => ['(', ')'],
+    'error'   => ['<', '>'],
+    'parent'  => [' ', ' '],
+);
+
 sub init {
     my $self = shift;
 
+    $self->{+COMPOSER} ||= Test2::Formatter::Test2::Composer->new;
+
+    $self->{+_ACTIVE_DISP} = '';
+
     $self->{+VERBOSE} = 1 unless defined $self->{+VERBOSE};
 
     $self->{+JOB_LENGTH} ||= 2;
 
-    unless ($self->{+IO}) {
-        open(my $io, '>&', STDOUT) or die "Can't dup STDOUT:  $!";
-        $self->{+IO} = $io;
-    }
-
-    my $io = $self->{+IO};
+    my $io = $self->{+IO} = clone_io($self->{+IO} || \*STDOUT) or die "Cannot get a filehandle: $!";
     $io->autoflush(1);
 
     $self->{+TTY} = -t $io unless defined $self->{+TTY};
 
+    my $use_color = ref($self->{+COLOR}) ? 1 : delete($self->{+COLOR});
+    $use_color = $self->{+TTY} unless defined $use_color;
+
     if ($self->{+TTY} && USE_ANSI_COLOR) {
         $self->{+SHOW_BUFFER} = 1 unless defined $self->{+SHOW_BUFFER};
-        $self->{+COLOR} = {
-            DEFAULT_COLOR(),
-            TAGS   => {DEFAULT_TAG_COLOR()},
-            FACETS => {DEFAULT_FACET_COLOR()},
-            JOBS   => [DEFAULT_JOB_COLOR()],
-        } unless defined $self->{+COLOR};
 
-        $self->{+JOB_COLORS} = {free => [@{$self->{+COLOR}->{JOBS}}]};
+        if ($use_color) {
+            $self->{+COLOR} = {
+                DEFAULT_COLOR(),
+                TAGS   => {DEFAULT_TAG_COLOR()},
+                FACETS => {DEFAULT_FACET_COLOR()},
+                JOBS   => [DEFAULT_JOB_COLOR()],
+            } unless defined $self->{+COLOR};
+
+            $self->{+JOB_COLORS} = {free => [@{$self->{+COLOR}->{JOBS}}]};
+        }
     }
     else {
         $self->{+SHOW_BUFFER} = 0 unless defined $self->{+SHOW_BUFFER};
@@ -183,6 +204,8 @@ sub write {
     my ($self, $e, $num, $f) = @_;
     $f ||= $e->facet_data;
 
+    my $should_show = $self->update_active_disp($f);
+
     $self->{+ECOUNT}++;
 
     $self->encoding($f->{control}->{encoding}) if $f->{control}->{encoding};
@@ -193,12 +216,12 @@ sub write {
 
     my $lines;
     if (!$self->{+VERBOSE}) {
-        unless ($depth) {
-            my $tree = $self->render_tree($f,);
-            $lines = $self->render_quiet($f, $tree);
+        if ($depth) {
+            $lines = [];
+        }
+        else {
+            $lines = $self->render_quiet($f);
         }
-
-        $lines ||= [];
     }
     elsif ($depth) {
         my $tree = $self->render_tree($f, '>');
@@ -209,12 +232,21 @@ sub write {
         $lines = $self->render_event($f, $tree);
     }
 
+    $should_show ||= $lines && @$lines;
+    unless ($should_show || $self->{+VERBOSE}) {
+        if (my $last = $self->{last_rendered}) {
+            $self->{last_rendered} = time;
+            return if time - $last < 0.2;
+        }
+        else {
+            $self->{last_rendered} = time;
+        }
+    }
+
     my $job_id = $f->{harness}->{job_id};
     push @{$self->{+JOB_COLORS}->{free}} => delete $self->{+JOB_COLORS}->{used}->{$job_id}
         if $job_id && $f->{harness_job_end};
 
-    return unless ($lines && @$lines) || !$self->{+VERBOSE};
-
     my $io = $self->{+IO};
     if ($self->{+_BUFFERED}) {
         print $io "\r\e[K";
@@ -228,60 +260,98 @@ sub write {
             $self->{+_BUFFERED} = 1;
         }
     }
-    elsif ($depth) {
+    elsif ($depth && $lines && @$lines) {
         print $io $lines->[0];
         $self->{+_BUFFERED} = 1;
     }
     else {
         print $io $_, "\n" for @$lines;
     }
+}
 
-    $io->flush;
+sub update_active_disp {
+    my $self = shift;
+    my ($f) = @_;
+
+    my $should_show = 0;
+
+    if ($f->{harness_job_launch}) {
+        my $job = $f->{harness_job};
+        $self->{+ACTIVE_FILES}->{File::Spec->abs2rel($job->{file})} = $job->{job_id};
+        $should_show = 1;
+    }
+
+    if ($f->{harness_job_end}) {
+        my $file = $f->{harness_job_end}->{file};
+        delete $self->{+ACTIVE_FILES}->{File::Spec->abs2rel($file)};
+        $should_show = 1;
+    }
+
+    return 0 unless $should_show;
+
+    my $active = $self->{+ACTIVE_FILES};
+
+    return $self->{+_ACTIVE_DISP} = '' unless $active && keys %$active;
+
+    my $str .= " (";
+    {
+        no warnings 'numeric';
+        $str .= join('  ' => map { m{([^/]+)$}; "$active->{$_}:$1" || "$active->{$_}:$_" } sort { ($active->{$a} || 0) <=> ($active->{$b} || 0) or $a cmp $b } keys %$active);
+    }
+    $str .= ")";
+
+    $self->{+_ACTIVE_DISP} = $str;
+
+    return 1;
 }
 
 sub render_ecount {
     my $self = shift;
-    return "Events seen: " . $self->{+ECOUNT};
+
+    my $str = "Events seen: $self->{+ECOUNT} $self->{+_ACTIVE_DISP}";
+
+    my $max = term_size() || 80;
+    $str = substr($str, 0, $max - 8) . " ...)" if length($str) > $max;
+
+    return $str;
 }
 
 sub render_buffered_event {
     my $self = shift;
     my ($f, $tree) = @_;
 
-    return [$self->render_halt($f, $tree)] if $f->{control}->{halt};
-    return [$self->render_assert($f, $tree)] if $f->{assert};
-    return [$self->render_errors($f, $tree)] if $f->{errors};
-    return [$self->render_plan($f, $tree)] if $f->{plan};
-    return [$self->render_info($f, $tree)] if $f->{info};
-    return [$self->render_times($f, $tree)] if $f->{times};
+    my $comp = $self->{+COMPOSER}->render_one_line($f) or return;
 
-    return [$self->render_about($f, $tree)] if $f->{about};
-
-    return;
+    return unless @$comp;
+    return [$self->build_line($tree, @$comp)];
 }
 
 sub render_event {
     my $self = shift;
     my ($f, $tree) = @_;
 
-    my @out;
+    my $comps = $self->{+COMPOSER}->render_verbose($f);
 
-    push @out => $self->render_halt($f, $tree) if $f->{control}->{halt};
-    push @out => $self->render_plan($f, $tree) if $f->{plan};
+    my (@parent, @times);
 
-    if ($f->{assert}) {
-        push @out => $self->render_assert($f, $tree);
-        push @out => $self->render_debug($f, $tree) unless $f->{assert}->{pass} || $f->{assert}->{no_debug};
-        push @out => $self->render_amnesty($f, $tree) if $f->{amnesty} && ! $f->{assert}->{pass};
+    if ($f->{parent}) {
+        @parent = $self->render_parent($f, $tree);
+
+        if (@$comps && $comps->[-1]->[0] eq 'times') {
+            my $times = pop(@$comps);
+            @times = $self->build_line($tree, @$times);
+        }
     }
 
-    push @out => $self->render_info($f, $tree) if $f->{info};
-    push @out => $self->render_errors($f, $tree) if $f->{errors};
-    push @out => $self->render_parent($f, $tree) if $f->{parent};
-    push @out => $self->render_times($f, $tree) if $f->{times};
+    my @out;
 
-    push @out => $self->render_about($f, $tree)
-        if $f->{about} && !(@out || grep { $f->{$_} } qw/stop plan info nest assert/);
+    for my $comp (@$comps) {
+        my $ctree = $tree;
+        substr($ctree, -2, 2, '+~') if $comp->[0] eq 'assert' && $f->{parent};
+        push @out => $self->build_line($ctree, @$comp);
+    }
+
+    push @out => (@parent, @times);
 
     return \@out;
 }
@@ -292,23 +362,17 @@ sub render_quiet {
 
     my @out;
 
-    push @out => $self->render_halt($f, $tree) if $f->{control}->{halt};
-
-    if ($f->{assert} && !$f->{assert}->{pass} && !$f->{amnesty}) {
-        push @out => $self->render_assert($f, $tree);
-        push @out => $self->render_debug($f, $tree) unless $f->{assert}->{pass} || $f->{assert}->{no_debug};
-        push @out => $self->render_amnesty($f, $tree) if $f->{amnesty} && ! $f->{assert}->{pass};
+    my $comps = $self->{+COMPOSER}->render_brief($f);
+    for my $comp (@$comps) {
+        my $ctree = $tree ||= $self->render_tree($f);
+        substr($ctree, -2, 2, '+~') if $comp->[0] eq 'assert' && $f->{parent};
+        push @out => $self->build_line($ctree, @$comp);
     }
 
-    if ($f->{info}) {
-        my $if = { %$f, info => [grep { $_->{debug} || $_->{important} } @{$f->{info}}] };
-        push @out => $self->render_info($if, $tree) if @{$if->{info}};
+    if ($f->{parent} && !$f->{amnesty}) {
+        push @out => $self->render_parent($f, $tree ||= $self->render_tree($f), quiet => 1);
     }
 
-    push @out => $self->render_times($f, $tree) if $f->{times};
-    push @out => $self->render_errors($f, $tree) if $f->{errors};
-    push @out => $self->render_parent($f, $tree, quiet => 1) if $f->{parent} && !$f->{amnesty};
-
     return \@out;
 }
 
@@ -346,7 +410,7 @@ sub render_tree {
 
 sub build_line {
     my $self = shift;
-    my ($facet, $tag, $tree, $text, $ps, $pe) = @_;
+    my ($tree, $facet, $tag, $text) = @_;
 
     $tree ||= '';
     $tag  ||= '';
@@ -355,12 +419,14 @@ sub build_line {
 
     substr($tree, -2, 1, '+') if $facet eq 'assert';
 
-    my $max = term_size() || 80;
+    $tag = substr($tag, 0 - TAG_WIDTH, TAG_WIDTH) if length($tag) > TAG_WIDTH;
+
+    my $max = $self->{+TTY} ? (term_size() || 80) : undef;
     my $color = $self->{+COLOR};
     my $reset = $color ? $color->{reset} || '' : '';
     my $tcolor = $color ? $color->{TAGS}->{$tag} || $color->{FACETS}->{$facet} || '' : '';
 
-    ($ps, $pe) = ('[', ']') unless $ps;
+    my ($ps, $pe) = @{$FACET_TAG_BORDERS{$facet} || $FACET_TAG_BORDERS{default}};
 
     $tag = uc($tag);
     my $length = length($tag);
@@ -398,7 +464,7 @@ sub build_line {
 
     my @out;
     for my $line (@lines) {
-        if( length("$ps$tag$pe  $tree$line") > $max) {
+        if($max && length("$ps$tag$pe  $tree$line") > $max) {
             @out = ();
             last;
         }
@@ -427,121 +493,6 @@ sub build_line {
     );
 }
 
-sub render_times {
-    my $self = shift;
-    my ($f, $tree) = @_;
-
-    return $self->build_line('times', 'TIME', $tree, $f->{about}->{details});
-}
-
-sub render_halt {
-    my $self = shift;
-    my ($f, $tree) = @_;
-
-    return $self->build_line('control', 'HALT', $tree, $f->{control}->{details});
-}
-
-sub render_plan {
-    my $self = shift;
-    my ($f, $tree) = @_;
-
-    my $plan = $f->{plan};
-    return $self->build_line('plan', 'NO  PLAN', $tree, $f->{plan}->{details}) if $plan->{none};
-
-    if ($plan->{skip}) {
-        return $self->build_line('plan', 'SKIP ALL', $tree, $f->{plan}->{details})
-            if $f->{plan}->{details};
-
-        return $self->build_line('plan', 'SKIP ALL', $tree, "No reason given");
-    }
-
-    return $self->build_line('plan', 'PLAN', $tree, "Expected asserions: $f->{plan}->{count}");
-}
-
-sub render_assert {
-    my $self = shift;
-    my ($f, $tree) = @_;
-
-    substr($tree, -2, 2, '+~') if $f->{parent};
-
-    my $name = $f->{assert}->{details} || '<UNNAMED ASSERTION>';
-
-    return $self->build_line('assert', 'PASS', $tree, $name)
-        if $f->{assert}->{pass};
-
-    return $self->build_line('assert', '! PASS !', $tree, $name)
-        if $f->{amnesty} && @{$f->{amnesty}};
-
-    return $self->build_line('assert', 'FAIL', $tree, $name)
-}
-
-sub render_amnesty {
-    my $self = shift;
-    my ($f, $tree) = @_;
-
-    my %seen;
-    return map {
-        $seen{join '' => @{$_}{qw/tag details/}}++
-            ? ()
-            : $self->build_line('amnesty', $_->{tag}, $tree, $_->{details}, '{', '}');
-    } @{$f->{amnesty}};
-}
-
-sub render_debug {
-    my $self = shift;
-    my ($f, $tree) = @_;
-
-    my $name  = $f->{assert}->{details};
-    my $trace = $f->{trace};
-
-    my $debug;
-    if ($trace) {
-        $debug = $trace->{details};
-        if(!$debug && $trace->{frame}) {
-            my $frame = $trace->{frame};
-            $debug = "$frame->[1] line $frame->[2]";
-        }
-    }
-
-    $debug ||= "[No trace info available]";
-
-    chomp($debug);
-
-    return $self->build_line('trace', 'DEBUG', $tree, $debug);
-}
-
-sub render_info {
-    my $self = shift;
-    my ($f, $tree) = @_;
-
-    return map {
-        my $details = $_->{details} || '';
-
-        my $msg;
-        if (ref($details)) {
-            require Data::Dumper;
-            my $dumper = Data::Dumper->new([$details])->Indent(2)->Terse(1)->Useqq(1)->Sortkeys(1);
-            chomp($msg = $dumper->Dump);
-        }
-        else {
-            chomp($msg = $details);
-        }
-
-        $self->build_line('info', $_->{tag}, $tree, $details, '(', ')')
-    } @{$f->{info}};
-}
-
-sub render_about {
-    my $self = shift;
-    my ($f, $tree) = @_;
-
-    return unless $f->{about} && $f->{about}->{package} && $f->{about}->{details};
-
-    my $type = substr($f->{about}->{package}, 0 - TAG_WIDTH, TAG_WIDTH);
-
-    return $self->build_line('info', $type, $tree, $f->{about}->{details});
-}
-
 sub render_parent {
     my $self = shift;
     my ($f, $tree, %params) = @_;
@@ -558,35 +509,14 @@ sub render_parent {
     return unless @out;
 
     push @out => (
-        $self->build_line('parent', '', "$tree^", '', ' ', ' '),
+        $self->build_line("$tree^", 'parent', '', ''),
     );
 
     return @out;
 }
 
 
-sub render_errors {
-    my $self = shift;
-    my ($f, $tree) = @_;
-
-    return map {
-        my $details = $_->{details};
-
-        my $msg;
-        if (ref($details)) {
-            require Data::Dumper;
-            my $dumper = Data::Dumper->new([$details])->Indent(2)->Terse(1)->Useqq(1)->Sortkeys(1);
-            chomp($msg = $dumper->Dump);
-        }
-        else {
-            chomp($msg = $details);
-        }
-
-        $self->build_line('error', $_->{fail} ? 'FATAL' : 'ERROR', $tree, $details, '<', '>')
-    } @{$f->{errors}};
-}
-
-sub DESTORY {
+sub DESTROY {
     my $self = shift;
 
     my $io = $self->{+IO} or return;
