@@ -2,11 +2,14 @@ package Test2::Harness;
 use strict;
 use warnings;
 
-our $VERSION = '0.001006'; # TRIAL
+our $VERSION = '0.001051'; # TRIAL
 
 use Carp qw/croak/;
 use List::Util qw/sum/;
 use Time::HiRes qw/sleep time/;
+use Sys::Hostname qw/hostname/;
+use Test2::Harness::Util::JSON qw/encode_canon_json encode_json/;
+use Data::Dumper;
 
 use Test2::Harness::Util::Term qw/USE_ANSI_COLOR/;
 
@@ -20,9 +23,17 @@ use Test2::Harness::Util::HashBase qw{
     -active
     -live
     -jobs
+    -jobs_todo
     -event_timeout
     -post_exit_timeout
     -run_id
+
+    -email_from
+    -email_owner
+    -slack_url
+    -slack_fail
+    -slack_notify
+    -slack_log
 };
 
 sub init {
@@ -46,18 +57,22 @@ sub init {
 sub run {
     my $self = shift;
 
-    while (1) {
-        $self->{+CALLBACK}->() if $self->{+CALLBACK};
-        my $complete = $self->{+FEEDER}->complete;
-        $self->iteration();
-        last if $complete;
-        sleep 0.02;
+    {
+        while (1) {
+            $self->{+CALLBACK}->() if $self->{+CALLBACK};
+            my $complete = $self->{+FEEDER}->complete;
+            $self->iteration();
+            last if $complete;
+            sleep 0.02;
+        }
     }
 
-    my(@fail, @pass);
-    for my $job_id (sort keys %{$self->{+WATCHERS}}) {
-        my $watcher = $self->{+WATCHERS}->{$job_id};
+    $_->finish() for @{$self->{+RENDERERS}};
 
+    my $seen = 0;
+    my(@fail, @pass);
+    while (my ($job_id, $watcher) = each %{$self->{+WATCHERS}}) {
+        $seen++;
         if ($watcher->fail) {
             push @fail => $watcher->job;
         }
@@ -66,9 +81,15 @@ sub run {
         }
     }
 
+    my $lost;
+    if (my $want = $self->{+JOBS_TODO}) {
+        $lost = $want - $seen;
+    }
+
     return {
         fail => \@fail,
         pass => \@pass,
+        lost => $lost,
     }
 }
 
@@ -83,14 +104,16 @@ sub iteration {
 
         # Track active watchers in a second hash, this avoids looping over all
         # watchers each iteration.
-        for my $job_id (sort keys %{$self->{+ACTIVE}}) {
-            my $watcher = $self->{+ACTIVE}->{$job_id};
+        while(my ($job_id, $watcher) = each %{$self->{+ACTIVE}}) {
+            # Give it up to 5 seconds
+            my $killed = $watcher->killed || 0;
+            my $done = $watcher->complete || ($killed ? (time - $killed) > 5 : 0) || 0;
 
-            if ($watcher->complete) {
+            if ($done) {
                 $self->{+FEEDER}->job_completed($job_id);
                 delete $self->{+ACTIVE}->{$job_id};
             }
-            elsif($self->{+LIVE}) {
+            elsif($self->{+LIVE} && !$killed) {
                 push @events => $self->check_timeout($watcher);
             }
         }
@@ -100,18 +123,18 @@ sub iteration {
         last unless @events;
 
         for my $event (@events) {
-            my $job_id = $event->job_id;
+            my $job_id = $event->{job_id};
             next if $jobs && !$jobs->{$job_id};
 
             # Log first, before the watchers transform the events.
-            $_->log_event($event) for @{$self->{+LOGGERS}};
+            $_->log_raw_event($event) for @{$self->{+LOGGERS}};
 
+            my @delayed;
             if ($job_id) {
-                # This will transform the events, possibly by adding facets
                 my $watcher = $self->{+WATCHERS}->{$job_id};
 
                 unless ($watcher) {
-                    my $job = $event->facet_data->{harness_job}
+                    my $job = $event->{facet_data}->{harness_job}
                         or die "First event for job ($job_id) was not a job start!";
 
                     $watcher = Test2::Harness::Watcher->new(
@@ -124,30 +147,145 @@ sub iteration {
                     $self->{+ACTIVE}->{$job_id} = $watcher if $live;
                 }
 
-                my $f;
-                ($event, $f) = $watcher->process($event);
-
-                next unless $event;
+                # This will transform the events, possibly by adding facets,
+                # any return items are new events it produced.
+                @delayed = $watcher->process($event);
 
+                my $f = $event->{facet_data};
                 if ($f && $f->{harness_job_end}) {
                     $f->{harness_job_end}->{file} = $watcher->file;
-                    $f->{harness_job_end}->{fail} = $watcher->fail;
+                    $f->{harness_job_end}->{fail} = $watcher->fail ? 1 : 0;
 
                     my $plan = $watcher->plan;
                     $f->{harness_job_end}->{skip} = $plan->{details} || "No reason given" if $plan && !$plan->{count};
 
-                    push @{$f->{info}} => $watcher->fail_info_facet_list;
+                    push @{$f->{errors}} => $watcher->fail_error_facet_list;
+
+                    $self->send_notices($watcher, $event) if $watcher->fail;
+
+                    $watcher->clear_events();
                 }
             }
 
-            # Render it now that the watchers have done their thing.
-            $_->render_event($event) for @{$self->{+RENDERERS}};
+            for my $re (@delayed, $event) {
+                $_->log_processed_event($re) for @{$self->{+LOGGERS}};
+
+                my $rf = $re->{facet_data};
+                next if $rf->{harness_watcher}->{no_render};
+
+                # Render it now that the watchers have done their thing.
+                $_->render_event($re) for @{$self->{+RENDERERS}};
+            }
         }
     }
 
     return;
 }
 
+sub send_notices {
+    my $self = shift;
+    my ($watcher, $event) = @_;
+
+    return unless $self->{+EMAIL_OWNER} || $self->{+SLACK_FAIL} || $self->{+SLACK_NOTIFY};
+
+    my $file = $watcher->file;
+    my $events = $watcher->events;
+    my $log = "";
+    my $pretty = "";
+    my $host = hostname();
+
+    require Test2::Formatter::Test2;
+    open(my $fh, '>>', \$pretty) or die "Could not open pretty: $!";
+    print $fh <<"    EOT";
+Test Failed on $host: $file
+
+    EOT
+    my $renderer = Test2::Formatter::Test2->new(io => $fh, verbose => 100);
+
+    for my $e (@$events) {
+        $log .= encode_canon_json($e) . "\n";
+        $renderer->write($e);
+    }
+    $log .= encode_canon_json($event) . "\n";
+    $renderer->write($event);
+
+    require Test2::Harness::Util::TestFile;
+    my $tf = Test2::Harness::Util::TestFile->new(file => $watcher->file);
+
+    $self->send_slack_owners($tf, $log, $pretty);
+    $self->send_email_owners($tf, $log, $pretty);
+}
+
+sub send_slack_owners {
+    my $self = shift;
+    my ($tf, $log, $pretty) = @_;
+
+    return unless $self->{+SLACK_URL};
+
+    my @to = $tf->meta('slack');
+    push @to => @{$self->{+SLACK_FAIL}} if $self->{+SLACK_FAIL};
+    return unless @to;
+
+    require HTTP::Tiny;
+    my $ht = HTTP::Tiny->new();
+
+    my @attach = ({
+        fallback => 'Test Output',
+        pretext  => 'Test Output',
+        text => "```$pretty```",
+        mrkdwn_in => ['text'],
+    });
+
+    push @attach => {
+        fallback => 'Test Event Log',
+        pretext  => 'Test Event Log',
+        text     => $log,
+    } if $self->{+SLACK_LOG};
+
+    my $host = hostname();
+    my $file = $tf->file;
+
+    for my $dest (@to) {
+        my $r = $ht->post(
+            $self->{+SLACK_URL},
+            {
+                headers => {'content-type' => 'application/json'},
+                content => encode_json(
+                    {
+                        channel     => $dest,
+                        text        => "Test Failed on $host: $file",
+                        attachments => \@attach
+                    }
+                ),
+            },
+        );
+        warn "Failed to send slack message to '$dest'" unless $r->{success};
+    }
+}
+
+sub send_email_owners {
+    my $self = shift;
+    my ($tf, $log, $body) = @_;
+
+    my @to = $tf->meta('owner');
+
+    my $host     = hostname();
+    my $subject  = "Test Failure on $host";
+
+    my $mail = Email::Stuffer->to(@to);
+    $mail->from($self->{+EMAIL_FROM});
+    $mail->subject($subject);
+    $mail->html_body("<html><body><pre>$body</pre></body></html>");
+
+    $mail->attach(
+        $log,
+        content_type => 'application/x-json-stream',
+        filename => 'log.jsonl',
+    );
+
+    eval { $mail->send_or_die; 1 } or warn $@;
+}
+
 sub check_timeout {
     my $self = shift;
     my ($watcher) = @_;
@@ -155,10 +293,18 @@ sub check_timeout {
     my $stamp = time;
 
     return unless $watcher->job->use_timeout;
+    return if $watcher->killed;
+
+    my $last_poll = $self->{+FEEDER}->job_lookup->{$watcher->job->job_id}->last_poll() or return;
+    my $poll_delta = $stamp - $last_poll;
+
+    # If we have not polled recently then we cannot fault the job for having no
+    # events.
+    return if $poll_delta > 2;
 
     my $delta = $stamp - $watcher->last_event;
 
-    if (my $timeout = $self->{+EVENT_TIMEOUT}) {
+    if (my $timeout = $watcher->job->event_timeout || $self->{+EVENT_TIMEOUT}) {
         return $self->timeout($watcher, 'event', $timeout, <<"        EOT") if $delta >= $timeout;
 This happens if a test has not produced any events within a timeout period, but
 does not appear to be finished. Usually this happens when a test has frozen.
@@ -168,7 +314,7 @@ does not appear to be finished. Usually this happens when a test has frozen.
     # Not done if there is no exit
     return unless $watcher->has_exit;
 
-    if (my $timeout = $self->{+POST_EXIT_TIMEOUT}) {
+    if (my $timeout = $watcher->job->postexit_timeout || $self->{+POST_EXIT_TIMEOUT}) {
         return $self->timeout($watcher, 'post-exit', $timeout, <<"        EOT") if $delta >= $timeout;
 Sometimes a test will fork producing output in the child while the parent is
 allowed to exit. In these cases we cannot rely on the original process exit to
@@ -191,9 +337,13 @@ sub timeout {
 
     $msg .= <<"    EOT";
 
-You can turn this off on a per-test basis by adding the comment
+If you do not want this test to time-out you can turn timeouts off on a
+per-test basis by adding the following comment to the top of your test file.
+
 # HARNESS-NO-TIMEOUT
-to the top of your test file (but below the #! line).
+
+CAUTION: THIS IS ALMOST ALWAYS THE WRONG THING TO DO!
+         A test suite can hang forever when you turn timeouts off.
     EOT
 
     my @info = (
@@ -210,18 +360,24 @@ to the top of your test file (but below the #! line).
         run_id     => $self->{+RUN_ID},
         event_id   => "timeout-$type-$job_id",
         stamp      => time,
+        times      => [times],
         facet_data => {info => \@info},
     );
 
     return $event unless $self->{+LIVE};
     my $pid = $watcher->job->pid || 'NA';
 
+    if($type eq 'post-exit') {
+        $watcher->set_complete(1);
+        return;
+    }
+
     push @info => {
         details   => "Killing job: $job_id, PID: $pid",
         debug     => 1,
         important => 1,
         tag       => 'TIMEOUT',
-    } unless $type eq 'post-exit';
+    };
 
     return $event if $watcher->kill;
 
