@@ -2,7 +2,7 @@ package Test2::Harness::Event;
 use strict;
 use warnings;
 
-our $VERSION = '0.001006'; # TRIAL
+our $VERSION = '0.001051'; # TRIAL
 
 use Carp qw/confess/;
 use Time::HiRes qw/time/;
@@ -29,22 +29,10 @@ use Test2::Harness::Util::HashBase qw{
     -run_id
     -job_id
     -stamp
+    -times
+    processed
 };
 
-{
-    no warnings 'redefine';
-
-    sub facet_data {
-        my $self = shift;
-        my $data = $self->{+FACET_DATA};
-        $data->{harness}->{run_id}   = $self->{+RUN_ID};
-        $data->{harness}->{job_id}   = $self->{+JOB_ID};
-        $data->{harness}->{event_id} = $self->{+EVENT_ID};
-        delete $data->{facet_data};
-        return $data;
-    }
-}
-
 sub trace     { $_[0]->{+FACET_DATA}->{trace} }
 sub set_trace { confess "'trace' is a read only attribute" }
 
@@ -68,6 +56,11 @@ sub init {
     confess "'event_id' is a required attribute"
         unless defined $self->{+EVENT_ID};
 
+    $data->{harness}->{+RUN_ID}   = $self->{run_id}   unless defined $data->{harness}->{+RUN_ID};
+    $data->{harness}->{+JOB_ID}   = $self->{job_id}   unless defined $data->{harness}->{+JOB_ID};
+    $data->{harness}->{+EVENT_ID} = $self->{event_id} unless defined $data->{harness}->{+EVENT_ID};
+    delete $data->{facet_data};
+
     # Original trace wins.
     if (my $trace = delete $self->{+TRACE}) {
         $self->{+FACET_DATA}->{trace} ||= $trace;
