@@ -2,7 +2,7 @@ package Test2::Harness::Feeder::JSONL;
 use strict;
 use warnings;
 
-our $VERSION = '0.001006'; # TRIAL
+our $VERSION = '0.001051'; # TRIAL
 
 use Carp qw/croak/;
 
@@ -52,12 +52,20 @@ sub poll {
 
     my @out;
     while (my $line = $self->{+FILE}->read_line) {
+        my $watcher = delete $line->{facet_data}->{harness_watcher};
+        next if $watcher->{added_by_watcher};
+
         bless($line->{facet_data}->{harness_run}, 'Test2::Harness::Run')
             if $line->{facet_data}->{harness_run};
 
         bless($line->{facet_data}->{harness_job}, 'Test2::Harness::Job')
             if $line->{facet_data}->{harness_job};
 
+        # Strip out any previous harness errors
+        if (my $errors = $line->{facet_data}->{errors}) {
+            @$errors = grep {!$_->{from_harness}} @$errors;
+        }
+
         push @out => Test2::Harness::Event->new(%$line);
         last if $max && @out >= $max
     }
