@@ -2,11 +2,12 @@ package Test2::Harness::Feeder::Run;
 use strict;
 use warnings;
 
-our $VERSION = '0.001006'; # TRIAL
+our $VERSION = '0.001051'; # TRIAL
 
 use Carp qw/croak/;
-use Time::HiRes qw/time/;
+use Time::HiRes qw/time sleep/;
 use Scalar::Util qw/blessed/;
+use List::Util qw/first/;
 
 use Test2::Harness::Feeder::Job;
 use Test2::Harness::Run::Dir;
@@ -23,6 +24,9 @@ use Test2::Harness::Util::HashBase qw{
     -keep_dir
     -job_ids
     -seen_jobs
+    -tail
+
+    -batch
 };
 
 sub init {
@@ -40,6 +44,7 @@ sub init {
         $dir = $self->{+DIR} = Test2::Harness::Run::Dir->new(
             root   => $dir,
             run_id => $self->{+RUN}->run_id,
+            tail   => $self->{+TAIL},
         );
     }
 
@@ -60,6 +65,7 @@ sub _harness_event {
         event_id   => 'harness-' . ${$self->{+EVENT_COUNTER_REF}}++,
         run_id     => $run->run_id,
         stamp      => time,
+        times      => [times],
         facet_data => {@_},
     );
 }
@@ -83,32 +89,62 @@ sub poll {
     push @out => map { $self->_harness_event(0, info => [{tag => 'INTERNAL', debug => 1, details => $_}]) } $dir->err_poll;
 
     my @new_jobs;
-    for my $job ($dir->job_poll(1)) {
-        my $job_id = $job->job_id;
-        next if $self->{+JOB_IDS} && !$self->{+JOB_IDS}->{$job_id};
 
-        my $jfeed = Test2::Harness::Feeder::Job->new(
-            job_id => $job_id,
-            run_id => $self->{+RUN}->run_id,
-            complete => $self->{+RUNNER} ? 0 : 1,
-            dir => File::Spec->catdir($self->{+DIR}->root, $job_id),
-            keep_dir => $self->{+KEEP_DIR},
-            event_counter_ref => $self->{+EVENT_COUNTER_REF},
-        );
+    my $stop = 0;
+    while (!$stop) {
+        $stop = 1;
 
-        $self->{+JOB_LOOKUP}->{$job_id} = $jfeed;
-        push @{$self->{+_ACTIVE}} => $jfeed;
+        for my $job ($dir->job_poll(1)) {
+            my $job_id = $job->job_id;
+            if ($self->{+JOB_IDS} && !$self->{+JOB_IDS}->{$job_id}) {
+                $stop = 0;
+                next;
+            }
 
-        push @new_jobs => $self->_harness_event(
-            $job_id,
-            harness_job_launch => {stamp => time},
-            harness_job        => $job,
-        );
+            my $dir = File::Spec->catdir($self->{+DIR}->root, $job_id);
+            my $count = 0;
+            until(-d $dir) {
+                # Wait up to 30 seconds
+                if ($count++ > 1500) {
+                    warn "Directory '$dir' never appeared";
+                    last;
+                }
+                sleep 0.02;
+            }
+
+            next unless -d $dir;
+
+            my $jfeed = Test2::Harness::Feeder::Job->new(
+                job_id => $job_id,
+                run_id => $self->{+RUN}->run_id,
+                complete => $self->{+RUNNER} ? 0 : 1,
+                dir => File::Spec->catdir($self->{+DIR}->root, $job_id),
+                keep_dir => $self->{+KEEP_DIR},
+                event_counter_ref => $self->{+EVENT_COUNTER_REF},
+            );
+
+            $self->{+JOB_LOOKUP}->{$job_id} = $jfeed;
+            push @{$self->{+_ACTIVE}} => $jfeed;
+
+            push @new_jobs => $self->_harness_event(
+                $job_id,
+                harness_job_launch => {stamp => time},
+                harness_job        => $job,
+            );
+        }
     }
 
+    my $run_exit = $self->{+RUNNER} ? $self->{+RUNNER}->exited : 1;
+
     while (my $jfeed = shift @{$self->{+_ACTIVE}}) {
-        my $nmax = $max - @new_jobs - @out;
-        last if $nmax < 1;
+        $jfeed->set_runner_exited(1) if $run_exit;
+
+        my $nmax = $max ? $max - @new_jobs - @out : undef;
+
+        if ($max && $nmax < 1) {
+            unshift @$active => ($jfeed, @{$self->{+_ACTIVE}});
+            last;
+        }
 
         my @events;
         if(!eval { @events = $jfeed->poll($nmax); 1 }) {
@@ -143,20 +179,20 @@ sub poll {
 sub complete {
     my $self = shift;
 
+    # If runner exited with an error we need to be complete
+    my $runner = $self->{+RUNNER};
+    return 1 if $runner && $runner->exit;
+
     if (my $job_ids = $self->{+JOB_IDS}) {
-        return 1 unless grep { !$self->{+SEEN_JOBS}->{$_} } keys %$job_ids;
+        return 1 unless first { !$self->{+SEEN_JOBS}->{$_} } keys %$job_ids;
         return 0;
     }
 
-    my $runner = $self->{+RUNNER} or return 1;
-    my $exit = $runner->exit;
-
-    # If runner exited with an error we need to be complete
-    return 1 if $exit;
-
     return 0 if @{$self->{+_ACTIVE}};
-
-    return $self->{+DIR}->complete;
+    return 1 if $self->{+DIR}->complete();
+    return 1 unless $runner;
+    return 1 if $runner->exited;
+    return 0;
 }
 
 sub job_completed {
