@@ -2,7 +2,7 @@ package Test2::Harness::Job;
 use strict;
 use warnings;
 
-our $VERSION = '0.001006'; # TRIAL
+our $VERSION = '0.001051'; # TRIAL
 
 use Carp qw/croak/;
 
@@ -18,13 +18,16 @@ use Test2::Harness::Util::HashBase qw{
     -args
     -input
     -times
-    -chdir
     -load
     -load_import
+    -preload
 
     -use_fork
     -use_stream
     -use_timeout
+
+    -event_timeout
+    -postexit_timeout
 };
 
 sub init {
