@@ -2,17 +2,16 @@ package Test2::Harness::Job::Runner;
 use strict;
 use warnings;
 
-our $VERSION = '0.001006'; # TRIAL
+our $VERSION = '0.001051'; # TRIAL
 
 use Carp qw/croak confess/;
 use List::Util qw/first/;
-use IPC::Open3 qw/open3/;
 use Scalar::Util qw/openhandle/;
 use Test2::Util qw/clone_io pkg_to_file/;
 
 use File::Spec();
 
-use Test2::Harness::Util qw/open_file/;
+use Test2::Harness::Util qw/open_file fqmod/;
 
 use Test2::Harness::Util::HashBase qw{
     -via
@@ -28,16 +27,18 @@ sub init {
 
     croak "Invalid output directory '$dir'" unless -d $dir;
 
-    my $via = $self->{+VIA} ||= ['Open3'];
+    my $via = $self->{+VIA} ||= ['IPC'];
     croak "'via' must be an array reference"
         if !ref($via) || ref($via) ne 'ARRAY';
 }
 
 require Test2::Harness::Job::Runner::Open3;
+require Test2::Harness::Job::Runner::IPC;
 require Test2::Harness::Job::Runner::Fork;
 
 my %RUN_MAP = (
     Open3 => 'Test2::Harness::Job::Runner::Open3',
+    IPC   => 'Test2::Harness::Job::Runner::IPC',
     Fork  => 'Test2::Harness::Job::Runner::Fork',
 );
 
@@ -49,17 +50,10 @@ sub run {
     my $via;
 
     for my $item (@{$self->{+VIA}}) {
-        next if $item eq 'Fork' && !$job->use_fork;
         my $class = $RUN_MAP{$item};
 
         unless ($class) {
-            if ($item =~ m/^\+(.*)/) {
-                $class = $1;
-            }
-            else {
-                $class = __PACKAGE__ . "::$item";
-            }
-
+            $class = fqmod(__PACKAGE__, $item);
             my $file = pkg_to_file($class);
             my $ok   = eval { require $file; 1 };
             my $err  = $@;
@@ -74,17 +68,7 @@ sub run {
         next unless $class->viable($self);
         my @out;
 
-        my $chdir = $self->job->chdir;
-        my $orig = File::Spec->curdir();
-        chdir($chdir) if $chdir;
-
-        my $ok = eval { @out = $class->run($self); 1 };
-        my $err = $@;
-
-        chdir($orig) if $chdir;
-        die $err unless $ok;
-
-        return @out;
+        return $class->run($self);
     }
 
     croak "No viable run method found";
