@@ -2,11 +2,13 @@ package Test2::Harness::Job::Runner::Fork;
 use strict;
 use warnings;
 
-our $VERSION = '0.001006'; # TRIAL
+our $VERSION = '0.001051'; # TRIAL
 
+use POSIX;
 use Scalar::Util qw/openhandle/;
 use Test2::Util qw/clone_io CAN_REALLY_FORK pkg_to_file/;
 use Test2::Harness::Util qw/write_file/;
+use Test2::Harness::Util::IPC qw/swap_io/;
 
 sub viable {
     my $class = shift;
@@ -18,7 +20,10 @@ sub viable {
 
     my $job = $test->job;
 
-    return 0 if @{$job->switches};
+    return 0 if !$job->use_fork;
+
+    # -w switch is ok, otherwise it is a no-go
+    return 0 if grep { !m/\s*-w\s*/ } @{$job->switches};
 
     return 1;
 }
@@ -28,6 +33,9 @@ sub run {
     my ($test) = @_;
 
     my $job = $test->job;
+    my $preloads = $job->preload || [];
+
+    $_->pre_fork($job) for @$preloads;
 
     my $pid = fork();
     die "Failed to fork: $!" unless defined $pid;
@@ -35,15 +43,33 @@ sub run {
     # In parent
     return ($pid, undef) if $pid;
 
+    # In Child
+    my $file = $job->file;
+
+    # toggle -w switch late
+    $^W = 1 if grep { m/\s*-w\s*/ } @{$job->switches};
+
     $SIG{TERM} = 'DEFAULT';
     $SIG{INT} = 'DEFAULT';
     $SIG{HUP} = 'DEFAULT';
 
-    for my $mod (@{$job->load || []}) {
-        my $file = pkg_to_file($mod);
-        require $file;
+    my $env = $job->env_vars;
+    {
+        no warnings 'uninitialized';
+        $ENV{$_} = $env->{$_} for keys %$env;
     }
 
+    $ENV{T2_HARNESS_FORKED}  = 1;
+    $ENV{T2_HARNESS_PRELOAD} = 1;
+
+    my ($in_file, $out_file, $err_file, $event_file) = $test->output_filenames;
+
+    $0 = File::Spec->abs2rel($file);
+    $class->_reset_DATA($file);
+    @ARGV = ();
+
+    $_->post_fork($job) for @$preloads;
+
     my $importer = eval <<'    EOT' or die $@;
 package main;
 #line 0 "-"
@@ -61,20 +87,10 @@ sub { shift->import(@_) }
         $importer->($mod, @args);
     }
 
-    # In Child
-    my $file = $job->file;
-
-    my $env = $job->env_vars;
-    $ENV{$_} = $env->{$_} for keys %$env;
-
-    $ENV{T2_HARNESS_FORKED}  = 1;
-    $ENV{T2_HARNESS_PRELOAD} = 1;
-
-    my ($in_file, $out_file, $err_file, $event_file) = $test->output_filenames;
-
-    $0 = $file;
-    $class->_reset_DATA($file);
-    @ARGV = ();
+    for my $mod (@{$job->load || []}) {
+        my $file = pkg_to_file($mod);
+        require $file;
+    }
 
     # if FindBin is preloaded, reset it with the new $0
     FindBin::init() if defined &FindBin::init;
@@ -90,22 +106,21 @@ sub { shift->import(@_) }
 
     # Keep a copy of the old STDERR for a while so we can still report errors
     my $stderr = clone_io(\*STDERR);
-    my $die = sub { print $stderr @_; exit 255 };
 
     write_file($in_file, $job->input);
 
-    close(STDIN) or die "Could not close STDIN: $!";
-    open(STDIN, '<', $in_file) or die "Could not re-open STDIN";
-    die "New STDIN did not get fileno 0!" unless fileno(STDIN) == 0;
+    my $die = sub {
+        my @caller = caller;
+        my @caller2 = caller(1);
+        my $msg = "$_[0] at $caller[1] line $caller[2] ($caller2[1] line $caller2[2]).\n";
+        print $stderr $msg;
+        print STDERR $msg;
+        POSIX::_exit(127);
+    };
 
-    close(STDOUT) or die "Could not close STDOUT: $!";
-    open(STDOUT, '>', $out_file) or die "Could not re-open STDOUT";
-    die "New STDOUT did not get fileno 1!" unless fileno(STDOUT) == 1;
-
-    # Should get fileno 2
-    close(STDERR) or $die->("Could not close STDERR: $!");
-    open(STDERR, '>', $err_file) or $die->("Could not re-open STDOUT");
-    $die->("New STDERR did not get fileno 2!") unless fileno(STDERR) == 2;
+    swap_io(\*STDIN,  $in_file,  $die);
+    swap_io(\*STDOUT, $out_file, $die);
+    swap_io(\*STDERR, $err_file, $die);
 
     # avoid child processes sharing the same seed value as the parent
     srand();
@@ -115,7 +130,8 @@ sub { shift->import(@_) }
         Test2::API::test2_post_preload_reset();
     }
 
-    push @INC => @{$job->libs};
+    my %seen;
+    @INC = grep { !$seen{$_}++ } (@{$job->libs}, @INC);
 
     if ($job->use_stream) {
         $ENV{T2_FORMATTER} = 'Stream';
@@ -130,6 +146,8 @@ sub { shift->import(@_) }
 
     @ARGV = @{$job->args};
 
+    $_->pre_launch($job) for @$preloads;
+
     return (undef, $file);
 }
 
