@@ -2,11 +2,12 @@ package Test2::Harness::Job::Runner::Open3;
 use strict;
 use warnings;
 
-our $VERSION = '0.001006'; # TRIAL
+our $VERSION = '0.001051'; # TRIAL
 
 use IPC::Open3 qw/open3/;
-use Test2::Harness::Util qw/open_file write_file/;
+use Test2::Harness::Util qw/open_file write_file local_env/;
 use Test2::Util qw/pkg_to_file/;
+use Carp qw/cluck/;
 
 use File::Spec();
 
@@ -21,6 +22,32 @@ sub find_inc {
     return File::Spec->rel2abs($inc);
 }
 
+sub command_file {
+    my $class = shift;
+    my ($test) = @_;
+    return File::Spec->abs2rel($test->job->file);
+}
+
+sub command {
+    my $class = shift;
+    my ($test, $event_file) = @_;
+
+    my $job = $test->job;
+
+    my %seen;
+    return (
+        $^X,
+        (map { "-I$_" } grep {!$seen{$_}++} @{$job->libs}, $class->find_inc, @INC),
+        $ENV{HARNESS_PERL_SWITCHES} ? $ENV{HARNESS_PERL_SWITCHES} : (),
+        @{$job->switches},
+        (map {"-M$_"} @{$job->load_import || []}),
+        (map {"-m$_"} @{$job->load || []}),
+        $job->use_stream ? ("-MTest2::Formatter::Stream=file,$event_file") : (),
+        $job->times ? ('-MTest2::Plugin::Times') : (),
+        $class->command_file($test),
+        @{$job->args},
+    );
+}
 
 sub run {
     my $class = shift;
@@ -41,41 +68,15 @@ sub run {
         $job->use_stream ? (T2_FORMATTER => 'Stream') : (),
     };
 
-    my @cmd = (
-        $^X,
-        (map { "-I$_" } @{$job->libs}, $class->find_inc),
-        $ENV{HARNESS_PERL_SWITCHES} ? $ENV{HARNESS_PERL_SWITCHES} : (),
-        @{$job->switches},
-        (map {"-m$_"} @{$job->load || []}),
-        (map {"-M$_"} @{$job->load_import || []}),
-        $job->use_stream ? ("-MTest2::Formatter::Stream=file,$event_file") : (),
-        $job->times ? ('-MTest2::Plugin::Times') : (),
-        $job->file,
-        @{$job->args},
-    );
-
-    my $old;
-    for my $key (keys %$env) {
-        $old->{$key} = $ENV{$key} if exists $ENV{$key};
-        $ENV{$key} = $env->{$key};
-    }
+    my @cmd = $class->command($test, $event_file);
 
     my $pid;
-
-    my $ok = eval {
+    local_env $env => sub {
         $pid = open3(
             '<&' . fileno($in_fh), ">&" . fileno($out_fh), ">&" . fileno($err_fh),
             @cmd,
         );
-        1;
     };
-    my $err = $@;
-
-    for my $key (keys %$env) {
-        exists $old->{$key} ? $ENV{$key} = $old->{$key} : delete $ENV{$key};
-    }
-
-    die $@ unless $ok;
 
     return ($pid, undef);
 }
