@@ -2,11 +2,11 @@ package Test2::Harness::Logger::JSONL;
 use strict;
 use warnings;
 
-our $VERSION = '0.001006'; # TRIAL
+our $VERSION = '0.001051'; # TRIAL
 
 use IO::Handle;
 
-use Test2::Harness::Util::JSON qw/encode_canon_json/;
+use Test2::Harness::Util::JSON qw/encode_json/;
 
 BEGIN { require Test2::Harness::Logger; our @ISA = ('Test2::Harness::Logger') }
 use Test2::Harness::Util::HashBase qw/-fh -prefix/;
@@ -23,13 +23,19 @@ sub init {
     }
 }
 
-sub log_event {
+sub log_processed_event {
     my $self = shift;
     my ($event) = @_;
 
     my $fh = $self->{+FH};
     my $prefix = $self->{+PREFIX};
-    print $fh $prefix, encode_canon_json($event), "\n";
+    print $fh $prefix, encode_json($event), "\n";
+}
+
+sub finish {
+    my $self = shift;
+    close($self->{+FH});
+    $self->{+FH} = undef;
 }
 
 1;
